@@ -196,42 +196,34 @@ panel_context_menu_build_edition (PanelWidget *panel_widget,
 				  GtkWidget   *menu)
 {
 	GtkWidget *menuitem;
+	GtkWidget *image;
 
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-			GTK_ICON_SIZE_MENU,
-			gtk_image_new_from_stock (GTK_STOCK_ADD,
-						  GTK_ICON_SIZE_MENU),
-			_("_Add to Panel..."),
-			FALSE);
+	menuitem = gtk_image_menu_item_new_with_mnemonic (_("_Add to Panel..."));
+	image = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
+	gtk_widget_show (menuitem);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-
         g_signal_connect (G_OBJECT (menuitem), "activate",
 	      	       	  G_CALLBACK (panel_addto_present), panel_widget);
 
 	if (!panel_profile_id_lists_are_writable ())
 		gtk_widget_set_sensitive (menuitem, FALSE);
 
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-			GTK_ICON_SIZE_MENU,
-			gtk_image_new_from_stock (GTK_STOCK_PROPERTIES,
-						  GTK_ICON_SIZE_MENU),
-			_("_Properties"),
-			FALSE);
-
+	menuitem = gtk_image_menu_item_new_with_mnemonic (_("_Properties"));
+	image = gtk_image_new_from_stock (GTK_STOCK_PROPERTIES,
+					  GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
+	gtk_widget_show (menuitem);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 	g_signal_connect_swapped (menuitem, "activate",
 				  G_CALLBACK (panel_properties_dialog_present), 
 				  panel_widget->toplevel);
 
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem, 
-			GTK_ICON_SIZE_MENU,
-			gtk_image_new_from_stock (GTK_STOCK_DELETE,
-						  GTK_ICON_SIZE_MENU),
-			_("_Delete This Panel"),
-			FALSE);
+	menuitem = gtk_image_menu_item_new_with_mnemonic (_("_Delete This Panel"));
+	image = gtk_image_new_from_stock (GTK_STOCK_DELETE,
+					  GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
+	gtk_widget_show (menuitem);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 	g_signal_connect_swapped (G_OBJECT (menuitem), "activate",
 				  G_CALLBACK (panel_context_menu_delete_panel),
@@ -242,13 +234,10 @@ panel_context_menu_build_edition (PanelWidget *panel_widget,
 
 	add_menu_separator (menu);
 
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem, 
-			GTK_ICON_SIZE_MENU,
-			gtk_image_new_from_stock (GTK_STOCK_NEW,
-						  GTK_ICON_SIZE_MENU),
-			_("_New Panel"),
-			FALSE);
+	menuitem = gtk_image_menu_item_new_with_mnemonic (_("_New Panel"));
+	image = gtk_image_new_from_stock (GTK_STOCK_NEW, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
+	gtk_widget_show (menuitem);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (panel_context_menu_create_new_panel), 
@@ -264,6 +253,7 @@ panel_context_menu_create (PanelWidget *panel)
 {
 	GtkWidget *retval;
 	GtkWidget *menuitem;
+	GtkWidget *image;
 
 	if (panel->master_widget) {
 		gpointer    *pointer;
@@ -287,24 +277,19 @@ panel_context_menu_create (PanelWidget *panel)
 	if (!panel_lockdown_get_locked_down ())
 		panel_context_menu_build_edition (panel, retval);
 
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-			GTK_ICON_SIZE_MENU,
-			gtk_image_new_from_stock (GTK_STOCK_HELP,
-						  GTK_ICON_SIZE_MENU),
-			_("_Help"),
-			FALSE);
+	menuitem = gtk_image_menu_item_new_with_mnemonic (_("_Help"));
+	image = gtk_image_new_from_stock (GTK_STOCK_HELP, GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
+	gtk_widget_show (menuitem);
 	gtk_menu_shell_append (GTK_MENU_SHELL (retval), menuitem);
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (panel_context_menu_show_help), NULL);
 
-	menuitem = gtk_image_menu_item_new ();
-	setup_menuitem (menuitem,
-			GTK_ICON_SIZE_MENU,
-			gtk_image_new_from_stock (GTK_STOCK_ABOUT,
-						  GTK_ICON_SIZE_MENU),
-			_("A_bout Panels"),
-			FALSE);
+	menuitem = gtk_image_menu_item_new_with_mnemonic (_("A_bout Panels"));
+	image = gtk_image_new_from_stock (GTK_STOCK_ABOUT,
+					  GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
+	gtk_widget_show (menuitem);
 	gtk_menu_shell_append (GTK_MENU_SHELL (retval), menuitem);
 	g_signal_connect (menuitem, "activate",
 			  G_CALLBACK (panel_context_menu_show_about_dialog),
